/************************************************************************
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*       "EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.      *
*                                                                       *
*  Copyright (C) 1985, 1992 Lugaru Software Ltd.  All rights reserved.  *
*                                                                       *
* Limited permission is hereby granted to reproduce and modify this     *
* copyrighted material provided that the resulting code is used only in *
* conjunction with Lugaru products and that this notice is retained in  *
* any such reproduction or modification.                                *
************************************************************************/

//  
//  LINE CUT/COPY/PASTE ROUTINES -- BY MORGWN McCARTY
//  -------------------------------------------------
//
//  key mappings:
//      set_mark_normal()       reg_tab[ALT(' ')]
//      set_mark_line()         reg_tab[CTRL(' ')]
//      set_mark_rectangle()    reg_tab[CTRL(ALT(' '))]
//      delete_line()           reg_tab[ALT('d')]
//      copy_text()             reg_tab[ALT('w')]
//      cut_text()              reg_tab[CTRL('w')]
//      paste_text()            reg_tab[CTRL('y')]
//      paste_pop()             reg_tab[ALT('y')]
//
//  revisions:
//      > 12/7/94 9:37am --mm-- initial release
//

#include "eel.h"


user int region_highlight_duration = 35; // change this to suit your needs


command set_mark_normal() on reg_tab[ALT(' ')]
{             
    set_mark();
    _is_rectangle = 0;
    modify_region(1, MRTYPE, REGNORM);
}


command set_mark_line() on reg_tab[CTRL(' ')]
{           
    set_mark();
    _is_rectangle = 0;
    modify_region(1, MRTYPE, REGLINE);
}


command set_mark_rectangle() on reg_tab[CTRL(ALT(' '))]
{             
    set_mark();
    _is_rectangle = 1;
    modify_region(1, MRTYPE, REGRECT);
}


command delete_line() on reg_tab[ALT('d')]
{
    mark = point;
    modify_region(1, MRTYPE, REGLINE);
    do_copy_text(1);
}


command copy_text() on reg_tab[ALT('w')]
{
    do_copy_text(0);
}


command cut_text() on reg_tab[CTRL('w')]
{
    do_copy_text(1);
}


int do_copy_text(with_cut)
{
    int orig_point;                     // save point
    int orig_col;                       // and save column
    int pos1;                           // beginning of region
    int pos2;                           // end of region
    int at_end_of_buffer;               // flag

    // maybe call standard epsilon function
    if ((is_highlight_on()) && (modify_region(1, MRTYPE, -1) != REGLINE)) {
        if (with_cut) {
            kill_region();
        } else {
            copy_region();
        }
        return;
    }

    // save original point and column
    orig_point = point;
    orig_col = virtual_column();

    // normalize region
    if (is_highlight_on()) {            // multiple lines
        if (point < mark) {
            pos1 = point;
            pos2 = mark;
        } else {
            pos1 = mark;
            pos2 = point;
        }
    } else {                            // force single line if no highlighting
        pos1 = pos2 = point;
    }
    
    // find beginning of first line
    point = pos1;
    to_begin_line();
    pos1 = point;

    // find end of last line
    point = pos2;
    nl_forward();
    pos2 = point;
    at_end_of_buffer = (current_column() != 0); // we're at end of buffer!

    // perform the cut or copy
    if (with_cut) {
        check_modify(bufnum);
        do_save_kill(pos1, pos2);
    } else {
        prev_cmd = 0;
        do_save_only(pos1, pos2);
        this_cmd = 0;
        highlight_off();
    }

    // append a return if we are copying from the end of the buffer
    if (at_end_of_buffer) {
        buffer_printf(_cur_kill_buf, "\n");
    }

    // set this kill buffer flag to "line-mode"
    _kill_width[_num_kill] = -2;

    // restore row and column
    if (with_cut) {
        point = pos1;
        to_virtual_column(orig_col);
    } else {
        point = orig_point;
    }
}    


command paste_text() on reg_tab[CTRL('y')]
{
    paste_retrieve(0);
}


command paste_pop() on reg_tab[ALT('y')]
{
    paste_retrieve(1);
}


int _oldfrom = -1, _oldto = -1; /* region yanked last time */
#define UNDO_FLAG_KILL  1   /* bracket each yank in undo with these */
#define UNDO_FLAG_RETR  2   /* for yank-pop */

paste_retrieve(withpop)
{
    char *thisbuf;
    int orig_col;
    
    // about to modify buffer, maybe give error
    check_modify(bufnum);

    // ignore yanking to kill buffer
    thisbuf = bufname;
    if (thisbuf == _cur_kill_buf)
        return;

    // quit if no kill buffers used
    check_num_kill_buffers();
    if (_kill_width[_num_kill] == UNUSED)
        return;

    // save column
    orig_col = virtual_column();
    
    // zap last yanked region now if we're doing "yank-pop"
    if (withpop && !pop_retrieve())
        error("Must follow yank or yank-pop");

    // set header for undo stuff
    undo_flag = UNDO_FLAG_KILL;
    
    // yank the new region
    switch (_kill_width[_num_kill]) {

        // normal region
        case REGION:    
            _oldfrom = point;
            bufname = _cur_kill_buf;
            xfer(thisbuf, 0, size());
            bufname = thisbuf;
            flash_region(REGNORM);
            _oldto = point;       
            break;

        // lines
        case -2:
            to_begin_line();
            _oldfrom = point;
            bufname = _cur_kill_buf;
            xfer(thisbuf, 0, size());
            bufname = thisbuf;
            flash_region(REGNORM);
            to_virtual_column(orig_col);
            _oldto = point;
            break;

        // rectangle
        default:
            _oldfrom = point;
            bufname = _cur_kill_buf;
            rect_retrieve(thisbuf);
#if 1                                   // leave cursor at end of region!
            point = mark;
            _oldto = mark;
            mark = _oldfrom;
            flash_region(REGRECT);
#else
            point = _oldfrom;
            flash_region(REGRECT);
            _oldto = mark;
#endif
            break;
    }

    // make epsilon rebuild all of screen
    build_first = 1;
}


flash_region(region_type)
{
    TIMER ts;

    // highlight the region
    modify_region(1, MRTYPE, region_type);
    _highlight_control = 1;
    refresh();

    // wait for a bit
    time_begin(&ts, region_highlight_duration);
    while ((!time_done(&ts)) && (!char_avail()))
        ;

    // turn off highlighting and refresh screen
    _highlight_control = 0;
    refresh();
}
